package com.contentsquare.flutter

import android.content.Context
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** ContentsquarePlugin */
class ContentsquarePlugin : FlutterPlugin, MethodCallHandler {
  private lateinit var channel: MethodChannel
  private var context: Context? = null
  private var methodCallHandlerImpl = MethodCallHandlerImpl()

  override fun onAttachedToEngine(
      @NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding
  ) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "contentsquare")
    channel.setMethodCallHandler(this)
    context = flutterPluginBinding.applicationContext
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when (call.method) {
        "send" -> {
            methodCallHandlerImpl.send(call, result)
        }   
        "sendTransaction" -> {
            methodCallHandlerImpl.sendTransaction(call, result)
        }
        "sendDynamicVar" -> {
            methodCallHandlerImpl.sendDynamicVar(call, result)
        }
        "optIn" -> {
            methodCallHandlerImpl.optIn(call, result, context)
        }
        "optOut" -> {
            methodCallHandlerImpl.optOut(call, result, context)
        }
        "forgetMe" -> {
            methodCallHandlerImpl.forgetMe(call, result)
        }
        "getUserId" -> {
            methodCallHandlerImpl.getUserId(call, result)
        }
        "stopTracking" -> {
            methodCallHandlerImpl.stopTracking(call, result)
        }
        "resumeTracking" -> {
            methodCallHandlerImpl.resumeTracking(call, result)
        }
        else -> {
            result.notImplemented()
        }
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
    context = null
  }
}
