package com.contentsquare.flutter

import android.content.Context
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import com.contentsquare.android.Contentsquare
import com.contentsquare.android.api.model.Transaction

final class MethodCallHandlerImpl {
    fun send(call: MethodCall, result: Result) {
        val screenName = call.argument<String>("screenName") 
        if (screenName != null) {
            Contentsquare.send(screenName)
            result.success(0)
            return
        }
        result.error("MISSING_ARGUMENT", "Missing string for the screenName parameter", null)
    }

    fun sendTransaction(call: MethodCall, result: Result) {
        val price = call.argument<Double>("price") 
        val currency = call.argument<String>("currency") 
        val transactionId = call.argument<String>("transactionId") 

        if (price == null) {
            result.error("MISSING_ARGUMENT", "Missing double for the price parameter", null)
            return
        }
        if (currency == null) {
            result.error("MISSING_ARGUMENT", "Missing string for the currency parameter", null)
            return
        }
        if (transactionId == null) {
            Contentsquare.send(Transaction.builder(price!!.toFloat(), currency!!).build())
            result.success(0)
            return
        }
        Contentsquare.send(Transaction.builder(price!!.toFloat(), currency!!).id(transactionId!!).build())
        result.success(0)
    }

    fun sendDynamicVar(call: MethodCall, result: Result) {
        val key = call.argument<String>("key") 
        val stringValue = call.argument<String>("stringValue") 
        val intValue = call.argument<Integer>("intValue") 

        if (key == null) {
            result.error("MISSING_ARGUMENT", "Missing string for the key parameter", null)
            return
        }
        if (stringValue != null) {
            Contentsquare.send(key!!, stringValue!!);
            result.success(0)
            return
        }
        if (intValue != null) {
            Contentsquare.send(key!!, intValue!!.toLong());
            result.success(0)
            return
        }
        result.error("MISSING_ARGUMENT", "Missing stringValue or intValue parameter", null)
    }

    fun optIn(call: MethodCall, result: Result, context: Context?) {
        if (context == null) {
            result.error("NO_CONTEXT", "Cannot call optIn without context, please wait for FlutterEngine to start. This can happen if you call the optIn method from the global scope or from outside a build method.", null)
            return
        }
        Contentsquare.optIn(context!!)
        result.success(0)
    }


    fun optOut(call: MethodCall, result: Result, context: Context?) {
        if (context == null) {
            result.error("NO_CONTEXT", "Cannot call optOut without context, please wait for FlutterEngine to start. This can happen if you call the optOut method from the global scope or from outside a build method.", null)
            return
        }
        Contentsquare.optOut(context!!)
        result.success(0)
    }

    fun forgetMe(call: MethodCall, result: Result) {
        Contentsquare.forgetMe()
        result.success(0)
    }

    fun getUserId(call: MethodCall, result: Result) {
        val userId = Contentsquare.getUserId()
        result.success(userId)
    }

    fun stopTracking(call: MethodCall, result: Result) {
        Contentsquare.stopTracking()
        result.success(0)
    }

    fun resumeTracking(call: MethodCall, result: Result) {
        Contentsquare.resumeTracking()
        result.success(0)
    }
}
