import Flutter
import UIKit
import ContentsquareModule

public class SwiftContentsquarePlugin: NSObject, FlutterPlugin {
  private var methodCallHandlerImpl = MethodCallHandlerImpl(); 

  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "contentsquare", binaryMessenger: registrar.messenger())
    let instance = SwiftContentsquarePlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
    case "send":
      methodCallHandlerImpl.send(call, result)
      break
    case "sendTransaction":
      methodCallHandlerImpl.sendTransaction(call, result)
      break
    case "sendDynamicVar":
      methodCallHandlerImpl.sendDynamicVar(call, result)
      break
    case "optIn":
      methodCallHandlerImpl.optIn(call, result)
      break
    case "optOut":
      methodCallHandlerImpl.optOut(call, result)
      break
    case "forgetMe":
      methodCallHandlerImpl.forgetMe(call, result)
      break
    case "getUserId":
      methodCallHandlerImpl.getUserId(call, result)
      break
    case "stopTracking":
      methodCallHandlerImpl.stopTracking(call, result)
      break
    case "resumeTracking":
      methodCallHandlerImpl.resumeTracking(call, result)
      break
    default:
      result(FlutterMethodNotImplemented)
    }
  }
}


class MethodCallHandlerImpl {
  public func send(_ call: FlutterMethodCall, _ result: FlutterResult) {
    let arguments: [String : Any] = call.arguments as? [String : Any] ?? [:]
    guard let screenName = arguments["screenName"] as? String else {
      result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing string for the screenName parameter",
        details: nil)
      )
      return
    }

    Contentsquare.send(screenViewWithName: screenName)
    result(nil)
  }

  public func sendTransaction(_ call: FlutterMethodCall, _ result: FlutterResult) {
    let arguments: [String : Any] = call.arguments as? [String : Any] ?? [:]

    let transactionId = arguments["transactionId"] as? String ?? nil;
    let price = arguments["price"] as? Float ?? nil;
    let currency = arguments["currency"] as? String ?? nil;

    if (price == nil) {
      result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing double for the price parameter",
        details: nil)
      )
      return
    }
    if (currency == nil) {
      result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing string for the currency parameter",
        details: nil)
      )
    }

    Contentsquare.send(transaction: CustomerTransaction(id: transactionId, value: price!, currency: currency!))
    result(nil)
  }

  public func sendDynamicVar(_ call: FlutterMethodCall, _ result: FlutterResult) {
    let arguments: [String : Any] = call.arguments as? [String : Any] ?? [:]

    let key = arguments["key"] as? String ?? nil;
    let stringValue = arguments["stringValue"] as? String ?? nil;
    let intValue = arguments["intValue"] as? UInt32 ?? nil;

    if (key == nil) {
      result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing string for the key parameter",
        details: nil)
      )
      return
    }
    do {
      if (stringValue != nil) {
        try Contentsquare.send(dynamicVar: DynamicVar(key: key!, value: stringValue!))
        result(nil)
        return
      }
      if (intValue != nil) {
        try Contentsquare.send(dynamicVar: DynamicVar(key: key!, value: intValue!))
        result(nil)
        return
      }
    } catch {
      result(FlutterError(
        code: "DYNAMIC_VAR_ERROR",
        message: "Error while sending the dynamic variable. Please make sure that you have defined the variable in the Contentsquare console",
        details: nil)
      )
      return
    }
    result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing stringValue or intValue parameter",
        details: nil)
      )
  }

  public func optIn(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.optIn()
    result(nil)
  }

  public func optOut(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.optOut()
    result(nil)
  }

  public func forgetMe(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.forgetMe()
    result(nil)
  }

  public func getUserId(_ call: FlutterMethodCall, _ result: FlutterResult) {
    result(Contentsquare.userID)
  }

  public func stopTracking(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.stopTracking()
    result(nil)
  }

  public func resumeTracking(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.resumeTracking()
    result(nil)
  }
}